local RW = LibStub("AceAddon-3.0"):GetAddon("Raid Watch")
local L = LibStub("AceLocale-3.0"):GetLocale("RW-2.0")

local LSM = LibStub("LibSharedMedia-3.0")

local Boss = RW.bossPrototype
local warningProt = {}
local mt = {__index = warningProt}

local Party = RW:GetPlugin("Party")

Boss.BITS = Boss.BITS or {}
Boss.BITS.WARNING = {
	NORMAL	= 0x00000001,
	CAST 	= 0x00000002,
	RUNAWAY = 0x00000004,
}

-- If text is a number this warning is considered not a custom one
-- If text is not a number then it is custom and it can be possibly sent as a table,
-- then the first value in the table will be the actual text, and the second oen will be the option text as its show in the GUI
-- 3rd value in the table will be the tooltip text
local custText = true
function Boss:_Warning(wType, text, icon, optionName, default, color, sound)
	if type(text) == "number" then 
		custText = false
		return self:_Warning(wType, GetSpellInfo(text), icon or text, optionName or text, icon, optionName, default, color, sound) end
	if type(icon) == "number" then 
		return self:_Warning(wType, text, select(3, GetSpellInfo(icon)), optionName or icon, default, color, sound) 
	end
	
	-- If option name is a number, then add it as a spell option and update it to the actual name
	-- Else add a gereral option
	if type(optionName) == "number" then
		self:AddSpellOption("General", optionName)
		optionName = RW.SpellName[optionName]
	else
		self:AddGeneralOption(optionName or text)
	end
	-- If no option name is found by now we use the text as default
	if not optionName then optionName = type(text) == "table" and text[1] or text end
	
	-- Extract the info from the text table for custom option
	local customOptName
	if type(text) == "table" then
		customOptName = text[2]
		text = text[1]
	elseif custText then
		customOptName = text
	end
	
	-- Do the magic to add all the bits and stuff
	self:AddDefaults(optionName, "WARNING", wType, default, customOptName)
	
	-- Add warnin specific data to the default
	local defaultOption = self.defaultOptions[optionName]	
	defaultOption.Colors = defaultOption.Colors or {}
	defaultOption.Sounds = defaultOption.Sounds or {}
	defaultOption.Flash = true
	defaultOption.Colors[customOptName or "WARNING"..optionName] = color
	if wType == "RUNAWAY" and not sound 
		then defaultOption.Sounds[customOptName or "WARNING"..optionName]  = "RW: RunAwayLittleGirl"
	else
		defaultOption.Sounds[customOptName or "WARNING"..optionName] = "RW: Tata"
	end
	
	local obj = setmetatable({
		type = wType,
		text = text,
		customText = custText,
		customOptName = customOptName,
		icon = icon,
		id = optionName,
		scheduled = {},
		mod = self,
	}, mt)
	
	self.warnings = self.warnings or {}
	tinsert(self.warnings, obj)
	
	custText = true
	
	return obj
end

function Boss:Warning(...)
	return self:_Warning("NORMAL", ...)
end

function Boss:RunawayWarning(...)
	return self:_Warning("RUNAWAY", ...)
end

function Boss:CastWarning(...)
	return self:_Warning("CAST", ...)
end

function Boss:CustomWarning(...)
	return self:_Warning("CUSTOM", ...)
end

function warningProt:Show(...)
	if bit.band(self.mod.db.profile[self.id].uses, RW.BITS.WARNING) ~= RW.BITS.WARNING then return end
	if self.customOptName then
		if not self.mod.db.profile[self.id]["WARNING"..self.customOptName] then return end
	elseif bit.band(self.mod.db.profile[self.id].WARNING, self.mod.BITS.WARNING[self.type]) ~= self.mod.BITS.WARNING[self.type] then return end
		
	local text
	if self.customText then			
		text = RW:Format(self.text, ...)		
	else
		text = RW:Format(L["FORMAT_WARNING_"..self.type], self.text, ...)
	end
	local db = self.mod.db.profile
	self.mod:SendMessage("RW_Warning", text, self.icon, db[self.id].Colors[self.customOptName or "WARNING"..self.id])
	-- Play sound
	if self.mod.db.profile[self.id].Sounds then 
		PlaySoundFile(LSM:Fetch("sound", db[self.id].Sounds[self.customOptName or "WARNING"..self.id])) 
	end
	-- Flash the screen
	if self.mod.db.profile[self.id].Flash then
		self.mod:SendMessage("RW_Flash", db[self.id].Colors[self.customOptName or "WARNING"..self.id])
	end
end

local function showMessage(message, ...)
	message:Show(...)
end

function warningProt:Schedule(stime, info, ...)
	self.mod:Schedule("WARNING"..self.id..(info or ""), stime, showMessage, self, info, ...)
	tinsert(self.scheduled, "WARNING"..self.id..(info or ""))
end

function warningProt:ScheduleRepeating(stime, now, info, ...)
	self.mod:ScheduleRepeating("WARNING"..self.id..(info or ""), stime, showMessage, now, self, info, ...)
	tinsert(self.scheduled, "WARNING"..self.id..(info or ""))
end

function warningProt:Unschedule(extraID)
	self.mod:Unschedule("WARNING"..self.id..(extraID or ""))
	RW:TRemove(self.scheduled, "WARNING"..self.id..(extraID or ""))
end

-- Options
Boss.optionGets = Boss.optionGets or {}
function Boss.optionGets.WARNING(self, option)
	local AceGUI = LibStub("AceGUI-3.0")
	local group = AceGUI:Create("InlineGroup")
	group:SetFullWidth(true)
	group:SetTitle("Options")
	
	-- For all warnings on this boss
	for i, warning in pairs(self.warnings) do
		-- If this warning belongs to the option
		if warning.id == option.text then
			local subGroup = AceGUI:Create("SimpleGroup")
			subGroup:SetFullWidth(true)
			subGroup:SetLayout("flow")
			
			local check = AceGUI:Create("CheckBox")
			if warning.customOptName then
				check:SetLabel(warning.customOptName)
				check:SetValue(self.db.profile[option.text]["WARNING"..warning.customOptName])
				check:SetUserData("optName", "WARNING"..warning.customOptName)
				check:SetCallback("OnValueChanged", self.optionHelpers.onCustomValueChanged)
			else
				check:SetLabel(L["WARNING_"..warning.type])
				check:SetValue(bit.band(self.db.profile[option.text].WARNING, Boss.BITS.WARNING[warning.type]) == Boss.BITS.WARNING[warning.type])
				check:SetUserData("type", "WARNING")
				check:SetUserData("subType", warning.type)
				check:SetCallback("OnValueChanged", self.optionHelpers.onValueChanged)
				check:SetCallback("OnEnter", self.optionHelpers.onOptionEnter)
			end		
			check:SetUserData("option", option)
			check:SetUserData("boss", self)		
			check:SetCallback("OnLeave", self.optionHelpers.onOptionLeave)
						
			local color = AceGUI:Create("ColorPicker")
			local colorvalue = self.db.profile[option.text].Colors[warning.customOptName or "WARNING"..warning.id]
			if colorvalue then 
				color:SetColor(unpack(colorvalue))
			end
			color:SetLabel("Color")
			color:SetUserData("option", option)
			color:SetUserData("boss", self)
			color:SetUserData("key", "Colors")
			color:SetUserData("subKey", warning.customOptName or "WARNING"..warning.id)
			color:SetCallback("OnValueConfirmed", self.optionHelpers.onValueChanged2)
			
			local drop = AceGUI:Create("LSM30_Sound")
			drop:SetList()
			--drop:SetList(AceGUIWidgetLSMlists.sound)
			drop:SetLabel("Sound")
			drop:SetUserData("option", option)
			drop:SetUserData("boss", self)
			drop:SetUserData("key", "Sounds")
			drop:SetUserData("subKey", warning.customOptName or "WARNING"..warning.id)
			drop:SetCallback("OnValueChanged", self.optionHelpers.onValueChanged2)
			drop:SetValue(self.db.profile[option.text].Sounds[warning.customOptName or "WARNING"..warning.id])

			local flash = AceGUI:Create("CheckBox")
			flash:SetValue(self.db.profile[option.text].Flash)
			flash:SetUserData("option", option)
			flash:SetUserData("boss", self)
			flash:SetUserData("optName", "Flash")
			flash:SetCallback("OnValueChanged", self.optionHelpers.onCustomValueChanged)
			
			subGroup:AddChild(check)
			subGroup:AddChild(color)
			subGroup:AddChild(drop)
			subGroup:AddChild(flash)
			
			group:AddChild(subGroup)
		end
	end
	return group
end 